# -*- coding: utf-8 -*-
'''
:codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import win_servermanager

# Globals
win_servermanager.__salt__ = {}
win_servermanager.__grains__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinServermanagerTestCase(TestCase):
    '''
    Test cases for salt.modules.win_servermanager
    '''
    def test_list_available(self):
        '''
        Test win_servermanager.list_available
        '''
        mock = MagicMock(return_value='')
        with patch.dict(win_servermanager.__salt__, {'cmd.shell': mock}):
            self.assertEqual(win_servermanager.list_available(), '')

    def test_list_installed(self):
        '''
        Test win_servermanager.list_installed
        '''
        mock = MagicMock(return_value=[{'Installed': True,
                                        'Name': 'Spongebob',
                                        'DisplayName': 'Square Pants'},
                                       {'Installed': False,
                                        'Name': 'Patrick',
                                        'DisplayName': 'Plankton'}])
        with patch.object(win_servermanager, '_pshell_json', mock):
            expected = {'Spongebob': 'Square Pants'}
            self.assertDictEqual(win_servermanager.list_installed(), expected)

    def test_install(self):
        '''
        Test win_servermanager.install
        '''
        mock = MagicMock(return_value={'ExitCode': 0,
                                       'Success': True,
                                       'FeatureResult':
                                           [{'DisplayName': 'Spongebob',
                                             'RestartNeeded': False}]})
        with patch.dict(
                win_servermanager.__grains__, {'osversion': '10.0.15130'}):
            with patch.object(win_servermanager, '_pshell_json', mock):
                expected = {'ExitCode': 0,
                            'DisplayName': 'Spongebob',
                            'RestartNeeded': False,
                            'Success': True}
                self.assertDictEqual(
                    win_servermanager.install('Telnet-Client'), expected)

    def test_remove(self):
        '''
        Test win_servermanager.remove
        '''
        mock = MagicMock(return_value={'ExitCode': 0,
                                       'Success': True,
                                       'FeatureResult':
                                           [{'DisplayName': 'Spongebob',
                                             'RestartNeeded': False}]})
        with patch.dict(
                win_servermanager.__grains__, {'osversion': '10.0.15130'}):
            with patch.object(win_servermanager, '_pshell_json', mock):
                expected = {'ExitCode': 0,
                            'DisplayName': 'Spongebob',
                            'RestartNeeded': False,
                            'Success': True}
                self.assertDictEqual(
                    win_servermanager.remove('Telnet-Client'), expected)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(WinServermanagerTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Test States
===========

Provide test case states that enable easy testing of things to do with
 state calls, e.g. running, calling, logging, output filtering etc.

.. code-block:: yaml

    always-passes-with-any-kwarg:
      test.nop:
        - name: foo
        - something: else
        - foo: bar

    always-passes:
      test.succeed_without_changes:
        - name: foo

    always-fails:
      test.fail_without_changes:
        - name: foo

    always-changes-and-succeeds:
      test.succeed_with_changes:
        - name: foo

    always-changes-and-fails:
      test.fail_with_changes:
        - name: foo

    my-custom-combo:
      test.configurable_test_state:
        - name: foo
        - changes: True
        - result: False
        - comment: bar.baz

    is-pillar-foo-present-and-bar-is-int:
      test.check_pillar:
        - present:
            - foo
        - integer:
            - bar
'''
from __future__ import absolute_import

# Import Python libs
import logging
import random
from salt.state import _gen_tag
from salt.exceptions import SaltInvocationError

log = logging.getLogger(__name__)


def nop(name, **kwargs):
    '''
    A no-op state that does nothing. Useful in conjunction with the `use`
    requisite, or in templates which could otherwise be empty due to jinja
    rendering

    .. versionadded:: 2015.8.1
    '''
    return succeed_without_changes(name)


def succeed_without_changes(name):
    '''
    Returns successful.

    .. versionadded:: 2014.7.0

    name
        A unique string.
    '''
    ret = {
        'name': name,
        'changes': {},
        'result': True,
        'comment': 'Success!'
    }
    return ret


def fail_without_changes(name):
    '''
    Returns failure.

    .. versionadded:: 2014.7.0

    name:
        A unique string.
    '''
    ret = {
        'name': name,
        'changes': {},
        'result': False,
        'comment': 'Failure!'
    }

    if __opts__['test']:
        ret['result'] = False
        ret['comment'] = 'If we weren\'t testing, this would be a failure!'

    return ret


def succeed_with_changes(name):
    '''
    Returns successful and changes is not empty

    .. versionadded:: 2014.7.0

    name:
        A unique string.
    '''
    ret = {
        'name': name,
        'changes': {},
        'result': True,
        'comment': 'Success!'
    }

    # Following the docs as written here
    # http://docs.saltstack.com/ref/states/writing.html#return-data
    ret['changes'] = {
        'testing': {
            'old': 'Unchanged',
            'new': 'Something pretended to change'
        }
    }

    if __opts__['test']:
        ret['result'] = None
        ret['comment'] = ('If we weren\'t testing, this would be successful '
                          'with changes')

    return ret


def fail_with_changes(name):
    '''
    Returns failure and changes is not empty.

    .. versionadded:: 2014.7.0

    name:
        A unique string.
    '''
    ret = {
        'name': name,
        'changes': {},
        'result': False,
        'comment': 'Failure!'
    }

    # Following the docs as written here
    # http://docs.saltstack.com/ref/states/writing.html#return-data
    ret['changes'] = {
        'testing': {
            'old': 'Unchanged',
            'new': 'Something pretended to change'
        }
    }

    if __opts__['test']:
        ret['result'] = None
        ret['comment'] = ('If we weren\'t testing, this would be failed with '
                          'changes')

    return ret


def configurable_test_state(name, changes=True, result=True, comment=''):
    '''
    A configurable test state which determines its output based on the inputs.

    .. versionadded:: 2014.7.0

    name:
        A unique string.
    changes:
        Do we return anything in the changes field?
        Accepts True, False, and 'Random'
        Default is True
    result:
        Do we return successfully or not?
        Accepts True, False, and 'Random'
        Default is True
        If test is True and changes is True, this will be None.  If test is
        True and and changes is False, this will be True.
    comment:
        String to fill the comment field with.
        Default is ''
    '''
    ret = {
        'name': name,
        'changes': {},
        'result': False,
        'comment': comment
    }
    change_data = {
        'testing': {
            'old': 'Unchanged',
            'new': 'Something pretended to change'
        }
    }

    if changes == 'Random':
        if random.choice([True, False]):
            # Following the docs as written here
            # http://docs.saltstack.com/ref/states/writing.html#return-data
            ret['changes'] = change_data
    elif changes is True:
        # If changes is True we place our dummy change dictionary into it.
        # Following the docs as written here
        # http://docs.saltstack.com/ref/states/writing.html#return-data
        ret['changes'] = change_data
    elif changes is False:
        ret['changes'] = {}
    else:
        err = ('You have specified the state option \'Changes\' with'
               ' invalid arguments. It must be either '
               ' \'True\', \'False\', or \'Random\'')
        raise SaltInvocationError(err)

    if result == 'Random':
        # since result is a boolean, if its random we just set it here,
        ret['result'] = random.choice([True, False])
    elif result is True:
        ret['result'] = True
    elif result is False:
        ret['result'] = False
    else:
        raise SaltInvocationError('You have specified the state option '
                                  '\'Result\' with invalid arguments. It must '
                                  'be either \'True\', \'False\', or '
                                  '\'Random\'')

    if __opts__['test']:
        ret['result'] = True if changes is False else None
        ret['comment'] = 'This is a test' if not comment else comment

    return ret


def show_notification(name, text=None, **kwargs):
    '''
    Simple notification using text argument.

    .. versionadded:: 2015.8.0

    name
        A unique string.

    text
        Text to return in the comment.
    '''

    if not text:
        raise SaltInvocationError('Missing required argument text.')

    ret = {
        'name': name,
        'changes': {},
        'result': True,
        'comment': text
    }

    return ret


def mod_watch(name, sfun=None, **kwargs):
    ''''
    Call this function via a watch statement

    .. versionadded:: 2014.7.0

    Any parameters in the state return dictionary can be customized by adding
    the keywords ``result``, ``comment``, and ``changes``.

    .. code-block:: yaml

        this_state_will_return_changes:
          test.succeed_with_changes

        this_state_will_NOT_return_changes:
          test.succeed_without_changes

        this_state_is_watching_another_state:
          test.succeed_without_changes:
            - comment: 'This is a custom comment'
            - watch:
              - test: this_state_will_return_changes
              - test: this_state_will_NOT_return_changes

        this_state_is_also_watching_another_state:
          test.succeed_without_changes:
            - watch:
              - test: this_state_will_NOT_return_changes
    '''
    has_changes = []
    for req in __low__['__reqs__']['watch']:
        tag = _gen_tag(req)
        if __running__[tag]['changes']:
            has_changes.append('{state}: {__id__}'.format(**req))

    ret = {
        'name': name,
        'result': kwargs.get('result', True),
        'comment': kwargs.get('comment', 'Watch statement fired.'),
        'changes': kwargs.get('changes', {
            'Requisites with changes': has_changes,
        }),
    }
    return ret


def _check_key_type(key_str, key_type=None):
    '''
    Helper function to get pillar[key_str] and
    check if its type is key_type

    Returns None if the pillar key is missing.
    If present True or False depending on match
    of the values type.

    Can't check for None.
    '''
    value = __salt__['pillar.get'](key_str, None)
    if value is None:
        return None
    elif type(value) is not key_type:
        return False
    else:
        return True


def _if_str_then_list(listing):
    '''
    Checks if its argument is a list or a str.
    A str will be turned into a list with the
    str as its only element.
    '''
    if type(listing) is str:
        return [listing]
    elif type(listing) is not list:
        raise TypeError
    return listing


def check_pillar(name,
        present=None,
        boolean=None,
        integer=None,
        string=None,
        listing=None,
        dictionary=None,
        verbose=False):
    '''
    Checks the presence and, optionally, the type of
    given keys in Pillar.
    Supported kwargs for types are:
    - boolean (bool)
    - integer (int)
    - string (str)
    - listing (list)
    - dictionary (dict)

    Checking for None type pillars is not implemented yet.

    .. code-block:: yaml

        is-pillar-foo-present-and-bar-is-int:
          test.check_pillar:
            - present:
                - foo
            - integer:
                - bar
    '''
    if not (present or boolean or integer or string or
            listing or dictionary):
        raise SaltInvocationError('Missing required argument text.')

    present = present or []
    boolean = boolean or []
    integer = integer or []
    string = string or []
    listing = listing or []
    dictionary = dictionary or []

    ret = {
        'name': name,
        'changes': {},
        'result': True,
        'comment': ''
    }
    checks = {}
    fine = {}
    failed = {}
    # for those we don't check the type:
    present = _if_str_then_list(present)
    checks[None] = present
    # those should be bool:
    boolean = _if_str_then_list(boolean)
    checks[bool] = boolean
    # those should be int:

    # those should be integer:
    integer = _if_str_then_list(integer)
    checks[int] = integer
    # those should be str:
    string = _if_str_then_list(string)
    checks[str] = string
    # those should be list:
    listing = _if_str_then_list(listing)
    checks[list] = listing
    # those should be dict:
    dictionary = _if_str_then_list(dictionary)
    checks[dict] = dictionary

    for key_type, keys in checks.items():
        for key in keys:
            result = _check_key_type(key, key_type)
            if result is None:
                failed[key] = None
                ret['result'] = False
            elif not result:
                failed[key] = key_type
                ret['result'] = False
            elif verbose:
                fine[key] = key_type

    for key, key_type in failed.items():
        comment = 'Pillar key "{0}" '.format(key)
        if key_type is None:
            comment += 'is missing.\n'
        else:
            comment += 'is not {0}.\n'.format(key_type)
        ret['comment'] += comment

    if verbose and fine:
        comment = 'Those keys passed the check:\n'
        for key, key_type in fine.items():
            comment += '- {0} ({1})\n'.format(key, key_type)
        ret['comment'] += comment

    return ret



# -*- coding: utf-8 -*-
'''
Manage JBoss 7 Application Server via CLI interface

.. versionadded:: 2015.5.0

This state uses jboss-cli.sh script from JBoss installation and parses its output to determine execution result.

In order to run each state, jboss_config dictionary with the following properties must be passed:

.. code-block:: yaml

   jboss:
      cli_path: '/opt/jboss/jboss-7.0/bin/jboss-cli.sh'
      controller: 10.11.12.13:9999
      cli_user: 'jbossadm'
      cli_password: 'jbossadm'

If controller doesn't require password, then passing cli_user and cli_password parameters is not obligatory.

Example of application deployment:

.. code-block:: yaml

     application_deployed:
      jboss7.deployed:
       - artifact:
           artifactory_url: http://artifactory.intranet.example.com/artifactory
           repository: 'ext-release-local'
           artifact_id: 'webcomponent'
           group_id: 'com.company.application'
           packaging: 'war'
           version: '0.1'
           target_dir: '/tmp'
        - jboss_config:
           cli_path: '/opt/jboss/jboss-7.0/bin/jboss-cli.sh'
           controller: 10.11.12.13:9999
           cli_user: 'jbossadm'
           cli_password: 'jbossadm'

Since same dictionary with configuration will be used in all the states, it is much more convenient to move jboss configuration and other properties
to pillar. For example, configuration of jboss server, artifactory address and application version could be moved to pillars:

.. code-block:: yaml

     application_deployed:
      jboss7.deployed:
       - artifact:
           artifactory_url: {{ pillar['artifactory']['url'] }}
           repository: {{ pillar['artifactory']['repository'] }}
           artifact_id: 'webcomponent'
           group_id: 'com.company.application'
           packaging: 'war'
           version: {{ pillar['webcomponent-artifact']['version'] }}
           latest_snapshot: {{ pillar['webcomponent-artifact']['latest_snapshot'] }}
           repository: {{ pillar['webcomponent-artifact']['repository'] }}
       - jboss_config: {{ pillar['jboss'] }}


Configuration in pillars:

.. code-block:: yaml

   artifactory:
      url: 'http://artifactory.intranet.example.com/artifactory'
      repository: 'libs-snapshots-local'

   webcomponent-artifact:
      repository: 'libs-snapshots-local'
      latest_snapshot: True
      version: -1 #If latest_snapshot then version is ignored

For the sake of brevity, examples for each state assume that jboss_config is moved to pillars.


'''

# Import python libs
from __future__ import absolute_import
import time
import logging
import re
import traceback

# Import Salt libs
from salt.utils import dictdiffer
from salt.exceptions import CommandExecutionError

# Import 3rd-party libs
import salt.ext.six as six

log = logging.getLogger(__name__)


def datasource_exists(name, jboss_config, datasource_properties, recreate=False):
    '''
    Ensures that a datasource with given properties exist on the jboss instance.
    If datasource doesn't exist, it is created, otherwise only the properties that are different will be updated.

    name
        Datasource property name
    jboss_config
        Dict with connection properties (see state description)
    datasource_properties
        Dict with datasource properties
    recreate : False
        If set to True and datasource exists it will be removed and created again. However, if there are deployments that depend on the datasource, it will not me possible to remove it.

    Example:

    .. code-block:: yaml

        sampleDS:
          jboss7.datasource_exists:
           - recreate: False
           - datasource_properties:
               driver-name: mysql
               connection-url: 'jdbc:mysql://localhost:3306/sampleDatabase'
               jndi-name: 'java:jboss/datasources/sampleDS'
               user-name: sampleuser
               password: secret
               min-pool-size: 3
               use-java-context: True
           - jboss_config: {{ pillar['jboss'] }}

    '''
    log.debug(" ======================== STATE: jboss7.datasource_exists (name: %s) ", name)
    ret = {'name': name,
           'result': True,
           'changes': {},
           'comment': ''}

    has_changed = False
    ds_current_properties = {}
    ds_result = __salt__['jboss7.read_datasource'](jboss_config=jboss_config, name=name)
    if ds_result['success']:
        ds_current_properties = ds_result['result']
        if recreate:
            remove_result = __salt__['jboss7.remove_datasource'](jboss_config=jboss_config, name=name)
            if remove_result['success']:
                ret['changes']['removed'] = name
            else:
                ret['result'] = False
                ret['comment'] = 'Could not remove datasource. Stdout: '+remove_result['stdout']
                return ret

            has_changed = True  # if we are here, we have already made a change

            create_result = __salt__['jboss7.create_datasource'](jboss_config=jboss_config, name=name, datasource_properties=datasource_properties)
            if create_result['success']:
                ret['changes']['created'] = name
            else:
                ret['result'] = False
                ret['comment'] = 'Could not create datasource. Stdout: '+create_result['stdout']
                return ret

            read_result = __salt__['jboss7.read_datasource'](jboss_config=jboss_config, name=name)
            if read_result['success']:
                ds_new_properties = read_result['result']
            else:
                ret['result'] = False
                ret['comment'] = 'Could not read datasource. Stdout: '+read_result['stdout']
                return ret

        else:
            update_result = __salt__['jboss7.update_datasource'](jboss_config=jboss_config, name=name, new_properties=datasource_properties)
            if not update_result['success']:
                ret['result'] = False
                ret['comment'] = 'Could not update datasource. '+update_result['comment']
                # some changes to the datasource may have already been made, therefore we don't quit here
            else:
                ret['comment'] = 'Datasource updated.'

            read_result = __salt__['jboss7.read_datasource'](jboss_config=jboss_config, name=name)
            ds_new_properties = read_result['result']
    else:
        if ds_result['err_code'] == 'JBAS014807':  # ok, resource not exists:
            create_result = __salt__['jboss7.create_datasource'](jboss_config=jboss_config, name=name, datasource_properties=datasource_properties)
            if create_result['success']:
                read_result = __salt__['jboss7.read_datasource'](jboss_config=jboss_config, name=name)
                ds_new_properties = read_result['result']
                ret['comment'] = 'Datasource created.'
            else:
                ret['result'] = False
                ret['comment'] = 'Could not create datasource. Stdout: '+create_result['stdout']
        else:
            raise CommandExecutionError('Unable to handle error: {0}'.format(ds_result['failure-description']))

    if ret['result']:
        log.debug("ds_new_properties=%s", str(ds_new_properties))
        log.debug("ds_current_properties=%s", str(ds_current_properties))
        diff = dictdiffer.diff(ds_new_properties, ds_current_properties)

        added = diff.added()
        if len(added) > 0:
            has_changed = True
            ret['changes']['added'] = __format_ds_changes(added, ds_current_properties, ds_new_properties)

        removed = diff.removed()
        if len(removed) > 0:
            has_changed = True
            ret['changes']['removed'] = __format_ds_changes(removed, ds_current_properties, ds_new_properties)

        changed = diff.changed()
        if len(changed) > 0:
            has_changed = True
            ret['changes']['changed'] = __format_ds_changes(changed, ds_current_properties, ds_new_properties)

        if not has_changed:
            ret['comment'] = 'Datasource not changed.'

    return ret


def __format_ds_changes(keys, old_dict, new_dict):
    log.debug("__format_ds_changes(keys=%s, old_dict=%s, new_dict=%s)", str(keys), str(old_dict), str(new_dict))
    changes = ''
    for key in keys:
        log.debug("key=%s", str(key))
        if key in old_dict and key in new_dict:
            changes += key+':'+__get_ds_value(old_dict, key)+'->'+__get_ds_value(new_dict, key)+'\n'
        elif key in old_dict:
            changes += key+'\n'
        elif key in new_dict:
            changes += key+':'+__get_ds_value(new_dict, key)+'\n'
    return changes


def __get_ds_value(dct, key):
    log.debug("__get_value(dict,%s)", key)
    if key == "password":
        return "***"
    elif dct[key] is None:
        return 'undefined'
    else:
        return str(dct[key])


def bindings_exist(name, jboss_config, bindings):
    '''
    Ensures that given JNDI binding are present on the server.
    If a binding doesn't exist on the server it will be created.
    If it already exists its value will be changed.

    jboss_config:
        Dict with connection properties (see state description)
    bindings:
        Dict with bindings to set.

    Example:

    .. code-block:: yaml

            jndi_entries_created:
              jboss7.bindings_exist:
               - bindings:
                  'java:global/sampleapp/environment': 'DEV'
                  'java:global/sampleapp/configurationFile': '/var/opt/sampleapp/config.properties'
               - jboss_config: {{ pillar['jboss'] }}

    '''
    log.debug(" ======================== STATE: jboss7.bindings_exist (name: %s) ", name)
    log.debug('bindings='+str(bindings))
    ret = {'name': name,
           'result': True,
           'changes': {},
           'comment': 'Bindings not changed.'}

    has_changed = False
    for key in bindings:
        value = str(bindings[key])
        query_result = __salt__['jboss7.read_simple_binding'](binding_name=key, jboss_config=jboss_config)
        if query_result['success']:
            current_value = query_result['result']['value']
            if current_value != value:
                update_result = __salt__['jboss7.update_simple_binding'](binding_name=key, value=value, jboss_config=jboss_config)
                if update_result['success']:
                    has_changed = True
                    __log_binding_change(ret['changes'], 'changed', key, value, current_value)
                else:
                    raise CommandExecutionError(update_result['failure-description'])
        else:
            if query_result['err_code'] == 'JBAS014807':  # ok, resource not exists:
                create_result = __salt__['jboss7.create_simple_binding'](binding_name=key, value=value, jboss_config=jboss_config)
                if create_result['success']:
                    has_changed = True
                    __log_binding_change(ret['changes'], 'added', key, value)
                else:
                    raise CommandExecutionError(create_result['failure-description'])
            else:
                raise CommandExecutionError(query_result['failure-description'])

    if has_changed:
        ret['comment'] = 'Bindings changed.'
    return ret


def __log_binding_change(changes, type_, key, new, old=None):
    if type_ not in changes:
        changes[type_] = ''
    if old is None:
        changes[type_] += key + ':' + new + '\n'
    else:
        changes[type_] += key + ':' + old + '->' + new + '\n'


def deployed(name, jboss_config, artifact=None, salt_source=None):
    '''
    Ensures that the given application is deployed on server.

    jboss_config:
        Dict with connection properties (see state description)
    artifact:
        If set, the artifact will be fetched from artifactory. This is a Dict object with the following properties:
           - artifactory_url: Full url to artifactory instance, for example: http://artifactory.intranet.example.com/artifactory
           - repository: One of the repositories, for example: libs-snapshots, ext-release-local, etc..
           - artifact_id: Artifact ID of the artifact
           - group_id: Group ID of the artifact
           - packaging: war/jar/ear, etc...
           - version: Artifact version. If latest_snapshot is set to True, the value of this attribute will be ignored, and newest snapshot will be taken instead.
           - latest_snapshot: If set to True and repository is a snapshot repository it will automatically select the newest snapshot.
           - snapshot_version: Exact version of the snapshot (with timestamp). A snapshot version may have several builds and a way to differentiate is to provide a build timestamp.
           - target_dir: Temporary directory on minion where artifacts will be downloaded
    salt_source:
        If set, the artifact to be deployed will be fetched from salt master. This is a Dict object with the following properties:
           - source: File on salt master (eg. salt://application-web-0.39.war)
           - target_file: Temporary file on minion to save file to (eg. '/tmp/application-web-0.39.war')
           - undeploy: Regular expression to match against existing deployments. If any deployment matches the regular expression then it will be undeployed.

    The deployment consists of the following steps:

    * Fetch artifact (salt filesystem, artifact or filesystem on minion)
    * Check if same artifact is not deployed yet (perhaps with different version)
    * Undeploy the artifact if it is already deployed
    * Deploy the new artifact

    Examples:

    Deployment of a file from Salt file system:

    .. code-block:: yaml

        application_deployed:
          jboss7.deployed:
           - salt_source:
                source: salt://application-web-0.39.war
                target_file: '/tmp/application-web-0.39.war'
                undeploy: 'application-web-.*'
           - jboss_config: {{ pillar['jboss'] }}

    Here, application-web-0.39.war file is downloaded from Salt file system to /tmp/application-web-0.39.war file on minion.
    Existing deployments are checked if any of them matches 'application-web-.*' regular expression, and if so then it
    is undeployed before deploying the application. This is useful to automate deployment of new application versions.

    JBoss state is capable of deploying artifacts directly from Artifactory repository. Here are some examples of deployments:

    1) Deployment of released version of artifact from Artifactory.

    .. code-block:: yaml

            application_deployed:
              jboss7.deployed:
               - artifact:
                   artifactory_url: http://artifactory.intranet.example.com/artifactory
                   repository: 'ext-release-local'
                   artifact_id: 'webcomponent'
                   group_id: 'com.company.application'
                   packaging: 'war'
                   version: '0.1'
                   target_dir: '/tmp'
                - jboss_config: {{ pillar['jboss'] }}

    This performs the following operations:

    * Download artifact from artifactory. In the example above the artifact will be fetched from: http://artifactory.intranet.example.com/artifactory/ext-release-local/com/company/application/webcomponent/0.1/webcomponent-0.1.war
      As a rule, for released versions the artifacts are downloaded from: artifactory_url/repository/group_id_with_slashed_instead_of_dots/artifact_id/version/artifact_id-version.packaging'
      This follows artifactory convention for artifact resolution. By default the artifact will be downloaded to /tmp directory on minion.
    * Connect to JBoss via controller (defined in jboss_config dict) and check if the artifact is not deployed already. In case of artifactory
      it will check if any deployment's name starts with artifact_id value. If deployment already exists it will be undeployed
    * Deploy the downloaded artifact to JBoss via cli interface.

    2) Deployment of last updated version of given SNAPSHOT version of artifact from Artifactory.

    .. code-block:: yaml

        application_deployed:
          jboss7.deployed:
           - artifact:
               artifactory_url: http://artifactory.intranet.example.com/artifactory
               repository: 'ext-snapshot-local'
               artifact_id: 'webcomponent'
               group_id: 'com.company.application'
               packaging: 'war'
               version: '0.1-SNAPSHOT'
            - jboss_config: {{ pillar['jboss'] }}

    Deploying snapshot version involves an additional step of resolving the exact version of the artifact (including the timestamp), which
    is not necessary when deploying a release.
    In the example above first a request will be made to retrieve the update timestamp from:
    http://artifactory.intranet.example.com/artifactory/ext-snapshot-local/com/company/application/webcomponent/0.1-SNAPSHOT/maven-metadata.xml
    Then the artifact will be fetched from
    http://artifactory.intranet.example.com/artifactory/ext-snapshot-local/com/company/application/webcomponent/0.1-SNAPSHOT/webcomponent-RESOLVED_SNAPSHOT_VERSION.war

    .. note:: In order to perform a snapshot deployment you have to:

        * Set repository to a snapshot repository.
        * Choose a version that ends with "SNAPSHOT" string.
          Snapshot repositories have a different layout and provide some extra information that is needed for deployment of the last or a specific snapshot.

    3) Deployment of SNAPSHOT version (with exact timestamp) of artifact from Artifactory.

    If you need to deploy an exact version of the snapshot you may provide snapshot_version parameter.

    .. code-block:: yaml

        application_deployed:
          jboss7.deployed:
           - artifact:
               artifactory_url: http://artifactory.intranet.example.com/artifactory
               repository: 'ext-snapshot-local'
               artifact_id: 'webcomponent'
               group_id: 'com.company.application'
               packaging: 'war'
               version: '0.1-SNAPSHOT'
               snapshot_version: '0.1-20141023.131756-19'
            - jboss_config: {{ pillar['jboss'] }}


    In this example the artifact will be retrieved from:
    http://artifactory.intranet.example.com/artifactory/ext-snapshot-local/com/company/application/webcomponent/0.1-SNAPSHOT/webcomponent-0.1-20141023.131756-19.war

    4) Deployment of latest snapshot of artifact from Artifactory.

    .. code-block:: yaml

        application_deployed:
          jboss7.deployed:
           - artifact:
               artifactory_url: http://artifactory.intranet.example.com/artifactory
               repository: 'ext-snapshot-local'
               artifact_id: 'webcomponent'
               group_id: 'com.company.application'
               packaging: 'war'
               latest_snapshot: True
            - jboss_config: {{ pillar['jboss'] }}

    Instead of providing an exact version of a snapshot it is sometimes more convenient to get the newest version. If artifact.latest_snapshot
    is set to True, then the newest snapshot will be downloaded from Artifactory. In this case it is not necessary to specify version.
    This is particulary useful when integrating with CI tools that will deploy the current snapshot to the Artifactory.

    '''
    log.debug(" ======================== STATE: jboss7.deployed (name: %s) ", name)
    ret = {'name': name,
           'result': True,
           'changes': {},
           'comment': ''}

    comment = ''

    validate_success, validate_comment = __validate_arguments(jboss_config, artifact, salt_source)
    if not validate_success:
        return _error(ret, validate_comment)

    log.debug('artifact=%s', str(artifact))
    resolved_source, get_artifact_comment = __get_artifact(artifact, salt_source)
    log.debug('resolved_source=%s', resolved_source)
    log.debug('get_artifact_comment=%s', get_artifact_comment)

    comment = __append_comment(new_comment=get_artifact_comment, current_comment=comment)
    if resolved_source is None:
        return _error(ret, get_artifact_comment)

    find_success, deployment, find_comment = __find_deployment(jboss_config, artifact, salt_source)
    if not find_success:
        return _error(ret, find_comment)

    log.debug('deployment=%s', deployment)
    if deployment is not None:
        __salt__['jboss7.undeploy'](jboss_config, deployment)
        ret['changes']['undeployed'] = deployment

    deploy_result = __salt__['jboss7.deploy'](jboss_config=jboss_config, source_file=resolved_source)
    log.debug('deploy_result=%s', str(deploy_result))
    if deploy_result['success']:
        comment = __append_comment(new_comment='Deployment completed.', current_comment=comment)
        ret['comment'] = comment
        ret['changes']['deployed'] = resolved_source
    else:
        comment = __append_comment(new_comment='''Deployment failed\nreturn code={retcode}\nstdout='{stdout}'\nstderr='{stderr}'''.format(**deploy_result), current_comment=comment)
        return _error(ret, comment)

    return ret


def __validate_arguments(jboss_config, artifact, salt_source):
    result, comment = __check_dict_contains(jboss_config, 'jboss_config', ['cli_path', 'controller'])
    if artifact is None and salt_source is None:
        result = False
        comment = __append_comment('No salt_source or artifact defined', comment)
    if artifact:
        result, comment = __check_dict_contains(artifact, 'artifact', ['artifactory_url', 'repository', 'artifact_id', 'group_id', 'packaging'], comment, result)
        if 'latest_snapshot' in artifact and isinstance(artifact['latest_snapshot'], str):
            if artifact['latest_snapshot'] == 'True':
                artifact['latest_snapshot'] = True
            elif artifact['latest_snapshot'] == 'False':
                artifact['latest_snapshot'] = False
            else:
                result = False
                comment = __append_comment('Cannot convert jboss_config.latest_snapshot={0} to boolean'.format(artifact['latest_snapshot']), comment)
        if 'version' not in artifact and ('latest_snapshot' not in artifact or not artifact['latest_snapshot']):
            result = False
            comment = __append_comment('No version or latest_snapshot=True in artifact')
    if salt_source:
        result, comment = __check_dict_contains(salt_source, 'salt_source', ['source', 'target_file'], comment, result)

    return result, comment


def __find_deployment(jboss_config, artifact=None, salt_source=None):
    result = None
    success = True
    comment = ''
    deployments = __salt__['jboss7.list_deployments'](jboss_config)
    if artifact is not None:
        for deployment in deployments:
            if deployment.startswith(artifact['artifact_id']):
                if result is not None:
                    success = False
                    comment = "More than one deployment's name starts with {0}. \n" \
                              "For deployments from artifactory existing deployments on JBoss are searched to find one that starts with artifact_id.\n"\
                              "Existing deployments: {1}".format(artifact['artifact_id'], ",".join(deployments))
                else:
                    result = deployment
    elif salt_source is not None and salt_source['undeploy']:
        deployment_re = re.compile(salt_source['undeploy'])
        for deployment in deployments:
            if deployment_re.match(deployment):
                if result is not None:
                    success = False
                    comment = "More than one deployment matches regular expression: {0}. \n" \
                              "For deployments from Salt file system deployments on JBoss are searched to find one that matches regular expression in 'undeploy' parameter.\n" \
                              "Existing deployments: {1}".format(salt_source['undeploy'], ",".join(deployments))
                else:
                    result = deployment

    return success, result, comment


def __get_artifact(artifact, salt_source):
    resolved_source = None
    comment = None

    if artifact is None and salt_source is None:
        log.debug('artifact == None and salt_source == None')
        comment = 'No salt_source or artifact defined'
    elif isinstance(artifact, dict):
        log.debug('artifact from artifactory')
        try:
            fetch_result = __fetch_from_artifactory(artifact)
            log.debug('fetch_result={0}'.format(fetch_result))
        except Exception as exception:
            log.debug(traceback.format_exc())
            return None, exception

        if fetch_result['status']:
            resolved_source = fetch_result['target_file']
            comment = fetch_result['comment']
        else:
            comment = 'Cannot fetch artifact (artifactory comment:{0}) '.format(fetch_result['comment'])
    elif isinstance(salt_source, dict):
        log.debug('file from salt master')

        try:
            sfn, source_sum, comment_ = __salt__['file.get_managed'](
                name=salt_source['target_file'],
                template=None,
                source=salt_source['source'],
                source_hash=None,
                user=None,
                group=None,
                mode=None,
                saltenv=__env__,
                context=None,
                defaults=None,
                kwargs=None)

            manage_result = __salt__['file.manage_file'](
                name=salt_source['target_file'],
                sfn=sfn,
                ret=None,
                source=salt_source['source'],
                source_sum=source_sum,
                user=None,
                group=None,
                mode=None,
                saltenv=__env__,
                backup=None,
                makedirs=False,
                template=None,
                show_diff=True,
                contents=None,
                dir_mode=None)
            if manage_result['result']:
                resolved_source = salt_source['target_file']
            else:
                comment = manage_result['comment']

        except Exception as e:
            log.debug(traceback.format_exc())
            comment = 'Unable to manage file: {0}'.format(e)

    return resolved_source, comment


def __fetch_from_artifactory(artifact):
    target_dir = '/tmp'
    if 'temp_dir' in artifact:
        target_dir = artifact['temp_dir']

    if 'latest_snapshot' in artifact and artifact['latest_snapshot']:
        fetch_result = __salt__['artifactory.get_latest_snapshot'](artifactory_url=artifact['artifactory_url'],
                                                                   repository=artifact['repository'],
                                                                   group_id=artifact['group_id'],
                                                                   artifact_id=artifact['artifact_id'],
                                                                   packaging=artifact['packaging'],
                                                                   target_dir=target_dir)
    elif str(artifact['version']).endswith('SNAPSHOT'):
        if 'snapshot_version' in artifact:
            fetch_result = __salt__['artifactory.get_snapshot'](artifactory_url=artifact['artifactory_url'],
                                                                repository=artifact['repository'],
                                                                group_id=artifact['group_id'],
                                                                artifact_id=artifact['artifact_id'],
                                                                packaging=artifact['packaging'],
                                                                version=artifact['version'],
                                                                snapshot_version=artifact['snapshot_version'],
                                                                target_dir=target_dir)
        else:
            fetch_result = __salt__['artifactory.get_snapshot'](artifactory_url=artifact['artifactory_url'],
                                                                repository=artifact['repository'],
                                                                group_id=artifact['group_id'],
                                                                artifact_id=artifact['artifact_id'],
                                                                packaging=artifact['packaging'],
                                                                version=artifact['version'],
                                                                target_dir=target_dir)
    else:
        fetch_result = __salt__['artifactory.get_release'](artifactory_url=artifact['artifactory_url'],
                                                           repository=artifact['repository'],
                                                           group_id=artifact['group_id'],
                                                           artifact_id=artifact['artifact_id'],
                                                           packaging=artifact['packaging'],
                                                           version=artifact['version'],
                                                           target_dir=target_dir)
    return fetch_result


def reloaded(name, jboss_config, timeout=60, interval=5):
    '''
    Reloads configuration of jboss server.

    jboss_config:
        Dict with connection properties (see state description)
    timeout:
        Time to wait until jboss is back in running state. Default timeout is 60s.
    interval:
        Interval between state checks. Default interval is 5s. Decreasing the interval may slightly decrease waiting time
        but be aware that every status check is a call to jboss-cli which is a java process. If interval is smaller than
        process cleanup time it may easily lead to excessive resource consumption.

    This step performs the following operations:

    * Ensures that server is in running or reload-required state (by reading server-state attribute)
    * Reloads configuration
    * Waits for server to reload and be in running state

    Example:

    .. code-block:: yaml

        configuration_reloaded:
           jboss7.reloaded:
            - jboss_config: {{ pillar['jboss'] }}
    '''
    log.debug(" ======================== STATE: jboss7.reloaded (name: %s) ", name)
    ret = {'name': name,
           'result': True,
           'changes': {},
           'comment': ''}

    status = __salt__['jboss7.status'](jboss_config)
    if not status['success'] or status['result'] not in ('running', 'reload-required'):
        ret['result'] = False
        ret['comment'] = "Cannot reload server configuration, it should be up and in 'running' or 'reload-required' state."
        return ret

    result = __salt__['jboss7.reload'](jboss_config)
    if result['success'] or \
                    'Operation failed: Channel closed' in result['stdout'] or \
                    'Communication error: java.util.concurrent.ExecutionException: Operation failed' in result['stdout']:
        wait_time = 0
        status = None
        while (status is None or not status['success'] or status['result'] != 'running') and wait_time < timeout:
            time.sleep(interval)
            wait_time += interval
            status = __salt__['jboss7.status'](jboss_config)

        if status['success'] and status['result'] == 'running':
            ret['result'] = True
            ret['comment'] = 'Configuration reloaded'
            ret['changes']['reloaded'] = 'configuration'
        else:
            ret['result'] = False
            ret['comment'] = 'Could not reload the configuration. Timeout ({0} s) exceeded. '.format(timeout)
            if not status['success']:
                ret['comment'] = __append_comment('Could not connect to JBoss controller.', ret['comment'])
            else:
                ret['comment'] = __append_comment(('Server is in {0} state'.format(status['result'])), ret['comment'])
    else:
        ret['result'] = False
        ret['comment'] = 'Could not reload the configuration, stdout:'+result['stdout']

    return ret


def __check_dict_contains(dct, dict_name, keys, comment='', result=True):
    for key in keys:
        if key not in six.iterkeys(dct):
            result = False
            comment = __append_comment("Missing {0} in {1}".format(key, dict_name), comment)
    return result, comment


def __append_comment(new_comment, current_comment=''):
    return current_comment+'\n'+new_comment


def _error(ret, err_msg):
    ret['result'] = False
    ret['comment'] = err_msg
    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import win_update


class MockPyWinUpdater(object):
    '''
        Mock PyWinUpdater class
    '''
    def __init__(self):
        pass

    @staticmethod
    def SetCategories(arg):
        '''
            Mock SetCategories
        '''
        return arg

    @staticmethod
    def SetIncludes(arg):
        '''
            Mock SetIncludes
        '''
        return arg

    @staticmethod
    def GetInstallationResults():
        '''
            Mock GetInstallationResults
        '''
        return True

    @staticmethod
    def GetDownloadResults():
        '''
            Mock GetDownloadResults
        '''
        return True

    @staticmethod
    def SetSkips(arg):
        return True


@patch('salt.states.win_update.PyWinUpdater', MockPyWinUpdater)
@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinUpdateTestCase(TestCase):
    '''
        Validate the win_update state
    '''
    def test_installed(self):
        '''
            Test to install specified windows updates
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': ''}

        mock = MagicMock(side_effect=[['Saltstack', False, 5],
                                      ['Saltstack', True, 5],
                                      ['Saltstack', True, 5],
                                      ['Saltstack', True, 5]])
        with patch.object(win_update, '_search', mock):
            ret.update({'comment': 'Saltstack'})
            self.assertDictEqual(win_update.installed('salt'), ret)

            mock = MagicMock(side_effect=[['dude', False, 5],
                                          ['dude', True, 5],
                                          ['dude', True, 5]])
            with patch.object(win_update, '_download', mock):
                ret.update({'comment': 'Saltstackdude'})
                self.assertDictEqual(win_update.installed('salt'), ret)

                mock = MagicMock(side_effect=[['@Me', False, 5],
                                              ['@Me', True, 5]])
                with patch.object(win_update, '_install', mock):
                    ret.update({'comment': 'Saltstackdude@Me'})
                    self.assertDictEqual(win_update.installed('salt'), ret)

                    ret.update({'changes': True, 'result': True})
                    self.assertDictEqual(win_update.installed('salt'), ret)

    def test_downloaded(self):
        '''
            Test to cache updates for later install.
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': ''}

        mock = MagicMock(side_effect=[['Saltstack', False, 5],
                                      ['Saltstack', True, 5],
                                      ['Saltstack', True, 5]])
        with patch.object(win_update, '_search', mock):
            ret.update({'comment': 'Saltstack'})
            self.assertDictEqual(win_update.downloaded('salt'), ret)

            mock = MagicMock(side_effect=[['dude', False, 5],
                                          ['dude', True, 5]])
            with patch.object(win_update, '_download', mock):
                ret.update({'comment': 'Saltstackdude'})
                self.assertDictEqual(win_update.downloaded('salt'), ret)

                ret.update({'changes': True, 'result': True})
                self.assertDictEqual(win_update.downloaded('salt'), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(WinUpdateTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
unit tests for the cache runner
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.runners import cache
import salt.utils

cache.__opts__ = {}
cache.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class CacheTest(TestCase):
    '''
    Validate the cache runner
    '''
    def test_grains(self):
        '''
        test cache.grains runner
        '''
        mock_minion = ['Larry']
        mock_ret = {}
        self.assertEqual(cache.grains(minion=mock_minion), mock_ret)

        mock_data = 'grain stuff'
        mock_outputter = 'deprecated'

        class MockMaster(object):
            def __init__(self, *args, **kwargs):
                pass

            def get_minion_grains(self):
                return mock_data

        mock_ret = {'outputter': mock_outputter, 'data': mock_data}
        with patch.object(salt.utils.master, 'MasterPillarUtil', MockMaster):
            self.assertEqual(cache.grains(outputter=mock_outputter), mock_ret)

        mock_outputter = None
        with patch.object(salt.utils.master, 'MasterPillarUtil', MockMaster):
            self.assertEqual(cache.grains(outputter=mock_outputter), mock_data)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(CacheTest, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import win_groupadd

# Import Other Libs
# pylint: disable=W0611
try:
    import win32com
    import pythoncom
    import pywintypes
    HAS_WIN_LIBS = True
except ImportError:
    HAS_WIN_LIBS = False
# pylint: enable=W0611


win_groupadd.__context__ = {}
win_groupadd.__opts__ = {'test': False}


@skipIf(not HAS_WIN_LIBS, 'win_groupadd unit tests can only be run if win32com, pythoncom, and pywintypes are installed')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinGroupTestCase(TestCase):
    '''
    Test cases for salt.modules.win_groupadd
    '''
    # 'add' function tests: 1

    def test_add(self):
        '''
        Test if it add the specified group
        '''
        self.assertDictEqual(win_groupadd.add('foo'),
                             {'changes': [], 'name': 'foo', 'result': None,
                              'comment': 'The group foo already exists.'})

    # 'delete' function tests: 1

    def test_delete(self):
        '''
        Test if it remove the specified group
        '''
        self.assertDictEqual(win_groupadd.delete('foo'),
                             {'changes': [], 'name': 'foo', 'result': None,
                              'comment': 'The group foo does not exists.'})

    # 'info' function tests: 1

    def test_info(self):
        '''
        Test if it return information about a group.
        '''
        with patch(win_groupadd.win32.client, 'flag', None):
            self.assertDictEqual(win_groupadd.info('dc=salt'),
                                 {'gid': None,
                                  'members': ['dc=\\user1'],
                                  'passwd': None,
                                  'name': 'WinNT://./dc=salt,group'})

        with patch(win_groupadd.win32.client, 'flag', 1):
            self.assertFalse(win_groupadd.info('dc=salt'))

        with patch(win_groupadd.win32.client, 'flag', 2):
            self.assertFalse(win_groupadd.info('dc=salt'))

    # 'getent' function tests: 1

    def test_getent(self):
        '''
        Test if it return info on all groups
        '''
        with patch.dict(win_groupadd.__context__, {'group.getent': True}):
            self.assertTrue(win_groupadd.getent())

    # 'adduser' function tests: 1

    def test_adduser(self):
        '''
        Test if it add a user to a group
        '''
        with patch(win_groupadd.win32.client, 'flag', None):
            self.assertDictEqual(win_groupadd.adduser('dc=foo', 'dc=\\username'),
                                 {'changes': {'Users Added': ['dc=\\username']},
                                  'comment': '', 'name': 'dc=foo', 'result': True})

        with patch(win_groupadd.win32.client, 'flag', 1):
            comt = ('Failed to add dc=\\username to group dc=foo.  C')
            self.assertDictEqual(win_groupadd.adduser('dc=foo', 'dc=\\username'),
                                 {'changes': {'Users Added': []}, 'name': 'dc=foo',
                                  'comment': comt, 'result': False})

    # 'deluser' function tests: 1

    def test_deluser(self):
        '''
        Test if it remove a user to a group
        '''
        ret = {'changes': {'Users Removed': []},
               'comment': 'User dc=\\username is not a member of dc=foo',
               'name': 'dc=foo', 'result': None}

        self.assertDictEqual(win_groupadd.deluser('dc=foo', 'dc=\\username'),
                             ret)

    # 'members' function tests: 1

    def test_members(self):
        '''
        Test if it remove a user to a group
        '''
        comment = ['Failure accessing group dc=foo.  C']
        ret = {'name': 'dc=foo', 'result': False, 'comment': comment,
               'changes': {'Users Added': [], 'Users Removed': []}}

        with patch(win_groupadd.win32.client, 'flag', 2):
            self.assertDictEqual(win_groupadd.members
                                 ('dc=foo', 'dc=\\user1,dc=\\user2,dc=\\user3'),
                                 ret)

        with patch(win_groupadd.win32.client, 'flag', 1):
            comment = ['Failed to add dc=\\user2 to dc=foo.  C',
                       'Failed to remove dc=\\user1 from dc=foo.  C']
            ret.update({'comment': comment, 'result': False})
            self.assertDictEqual(win_groupadd.members('dc=foo', 'dc=\\user2'), ret)

        with patch(win_groupadd.win32.client, 'flag', None):
            comment = ['dc=foo membership is correct']
            ret.update({'comment': comment, 'result': None})
            self.assertDictEqual(win_groupadd.members('dc=foo', 'dc=\\user1'), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(WinGroupTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import
import os.path

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import linux_lvm
from salt.exceptions import CommandExecutionError

# Globals
linux_lvm.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LinuxLVMTestCase(TestCase):
    '''
    TestCase for the salt.modules.linux_lvm module
    '''

    def test_version(self):
        '''
        Tests LVM version info from lvm version
        '''
        mock = MagicMock(return_value='Library version : 1')
        with patch.dict(linux_lvm.__salt__, {'cmd.run': mock}):
            self.assertEqual(linux_lvm.version(), '1')

    def test_fullversion(self):
        '''
        Tests all version info from lvm version
        '''
        mock = MagicMock(return_value='Library version : 1')
        with patch.dict(linux_lvm.__salt__, {'cmd.run': mock}):
            self.assertDictEqual(linux_lvm.fullversion(),
                                 {'Library version': '1'})

    def test_pvdisplay(self):
        '''
        Tests information about the physical volume(s)
        '''
        mock = MagicMock(return_value={'retcode': 1})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.pvdisplay(), {})

        mock = MagicMock(return_value={'retcode': 0,
                                       'stdout': 'A:B:C:D:E:F:G:H:I:J:K'})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.pvdisplay(),
                                 {'A': {'Allocated Physical Extents': 'K',
                                        'Current Logical Volumes Here': 'G',
                                        'Free Physical Extents': 'J',
                                        'Internal Physical Volume Number': 'D',
                                        'Physical Extent Size (kB)': 'H',
                                        'Physical Volume (not) Allocatable': 'F',
                                        'Physical Volume Device': 'A',
                                        'Physical Volume Size (kB)': 'C',
                                        'Physical Volume Status': 'E',
                                        'Total Physical Extents': 'I',
                                        'Volume Group Name': 'B'}})

            mockpath = MagicMock(return_value='Z')
            with patch.object(os.path, 'realpath', mockpath):
                self.assertDictEqual(linux_lvm.pvdisplay(real=True),
                                     {'Z': {'Allocated Physical Extents': 'K',
                                            'Current Logical Volumes Here': 'G',
                                            'Free Physical Extents': 'J',
                                            'Internal Physical Volume Number': 'D',
                                            'Physical Extent Size (kB)': 'H',
                                            'Physical Volume (not) Allocatable': 'F',
                                            'Physical Volume Device': 'A',
                                            'Physical Volume Size (kB)': 'C',
                                            'Physical Volume Status': 'E',
                                            'Real Physical Volume Device': 'Z',
                                            'Total Physical Extents': 'I',
                                            'Volume Group Name': 'B'}})

    def test_vgdisplay(self):
        '''
        Tests information about the volume group(s)
        '''
        mock = MagicMock(return_value={'retcode': 1})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.vgdisplay(), {})

        mock = MagicMock(return_value={'retcode': 0,
                                       'stdout': 'A:B:C:D:E:F:G:H:I:J:K:L:M:N:O:P:Q'})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.vgdisplay(),
                                 {'A': {'Actual Physical Volumes': 'K',
                                        'Allocated Physical Extents': 'O',
                                        'Current Logical Volumes': 'F',
                                        'Current Physical Volumes': 'J',
                                        'Free Physical Extents': 'P',
                                        'Internal Volume Group Number': 'D',
                                        'Maximum Logical Volume Size': 'H',
                                        'Maximum Logical Volumes': 'E',
                                        'Maximum Physical Volumes': 'I',
                                        'Open Logical Volumes': 'G',
                                        'Physical Extent Size (kB)': 'M',
                                        'Total Physical Extents': 'N',
                                        'UUID': 'Q',
                                        'Volume Group Access': 'B',
                                        'Volume Group Name': 'A',
                                        'Volume Group Size (kB)': 'L',
                                        'Volume Group Status': 'C'}})

    def test__lvdisplay(self):
        '''
        Return information about the logical volume(s)
        '''
        mock = MagicMock(return_value={'retcode': 1})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.lvdisplay(), {})

        mock = MagicMock(return_value={'retcode': 0,
                                       'stdout': 'A:B:C:D:E:F:G:H:I:J:K:L:M'})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.lvdisplay(),
                                 {'A': {'Allocated Logical Extents': 'I',
                                        'Allocation Policy': 'J',
                                        'Current Logical Extents Associated': 'H',
                                        'Internal Logical Volume Number': 'E',
                                        'Logical Volume Access': 'C',
                                        'Logical Volume Name': 'A',
                                        'Logical Volume Size': 'G',
                                        'Logical Volume Status': 'D',
                                        'Major Device Number': 'L',
                                        'Minor Device Number': 'M',
                                        'Open Logical Volumes': 'F',
                                        'Read Ahead Sectors': 'K',
                                        'Volume Group Name': 'B'}})

    def test_pvcreate(self):
        '''
        Tests for set a physical device to be used as an LVM physical volume
        '''
        self.assertEqual(linux_lvm.pvcreate(''),
                         'Error: at least one device is required')

        self.assertRaises(CommandExecutionError, linux_lvm.pvcreate, 'A')

        pvdisplay = MagicMock(return_value=True)
        with patch('salt.modules.linux_lvm.pvdisplay', pvdisplay):
            with patch.object(os.path, 'exists', return_value=True):
                ret = {'stdout': 'saltines', 'stderr': 'cheese', 'retcode': 0, 'pid': '1337'}
                mock = MagicMock(return_value=ret)
                with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
                    self.assertEqual(linux_lvm.pvcreate('A', metadatasize=1000), True)

    def test_pvremove(self):
        '''
        Tests for remove a physical device being used as an LVM physical volume
        '''
        pvdisplay = MagicMock(return_value=False)
        with patch('salt.modules.linux_lvm.pvdisplay', pvdisplay):
            self.assertRaises(CommandExecutionError, linux_lvm.pvremove, 'A', override=False)

        pvdisplay = MagicMock(return_value=False)
        with patch('salt.modules.linux_lvm.pvdisplay', pvdisplay):
            mock = MagicMock(return_value=True)
            with patch.dict(linux_lvm.__salt__, {'lvm.pvdisplay': mock}):
                ret = {'stdout': 'saltines', 'stderr': 'cheese', 'retcode': 0, 'pid': '1337'}
                mock = MagicMock(return_value=ret)
                with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
                    self.assertEqual(linux_lvm.pvremove('A'), True)

    def test_vgcreate(self):
        '''
        Tests create an LVM volume group
        '''
        self.assertEqual(linux_lvm.vgcreate('', ''),
                         'Error: vgname and device(s) are both required')

        mock = MagicMock(return_value='A\nB')
        with patch.dict(linux_lvm.__salt__, {'cmd.run': mock}):
            with patch.object(linux_lvm, 'vgdisplay', return_value={}):
                self.assertDictEqual(linux_lvm.vgcreate('A', 'B'),
                                     {'Output from vgcreate': 'A'})

    def test_vgextend(self):
        '''
        Tests add physical volumes to an LVM volume group
        '''
        self.assertEqual(linux_lvm.vgextend('', ''),
                         'Error: vgname and device(s) are both required')

        mock = MagicMock(return_value='A\nB')
        with patch.dict(linux_lvm.__salt__, {'cmd.run': mock}):
            with patch.object(linux_lvm, 'vgdisplay', return_value={}):
                self.assertDictEqual(linux_lvm.vgextend('A', 'B'),
                                     {'Output from vgextend': 'A'})

    def test_lvcreate(self):
        '''
        Test create a new logical volume, with option
        for which physical volume to be used
        '''
        self.assertEqual(linux_lvm.lvcreate(None, None, 1, 1),
                         'Error: Please specify only one of size or extents')

        self.assertEqual(linux_lvm.lvcreate(None, None, None, None),
                         'Error: Either size or extents must be specified')

        mock = MagicMock(return_value='A\nB')
        with patch.dict(linux_lvm.__salt__, {'cmd.run': mock}):
            with patch.object(linux_lvm, 'lvdisplay', return_value={}):
                self.assertDictEqual(linux_lvm.lvcreate(None, None, None, 1),
                                     {'Output from lvcreate': 'A'})

    def test_vgremove(self):
        '''
        Tests to remove an LVM volume group
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(linux_lvm.__salt__, {'cmd.run': mock}):
            self.assertEqual(linux_lvm.vgremove('A'), 'A')

    def test_lvremove(self):
        '''
        Test to remove a given existing logical volume
        from a named existing volume group
        '''
        mock = MagicMock(return_value='A')
        with patch.dict(linux_lvm.__salt__, {'cmd.run': mock}):
            self.assertEqual(linux_lvm.lvremove('', ''), 'A')

    def test_lvresize(self):
        '''
        Test to return information about the logical volume(s)
        '''
        mock = MagicMock(return_value={'retcode': 1})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.lvresize(1, 'a'), {})

        mock = MagicMock(return_value={'retcode': 0})
        with patch.dict(linux_lvm.__salt__, {'cmd.run_all': mock}):
            self.assertDictEqual(linux_lvm.lvresize(1, 'a'), {})

if __name__ == '__main__':
    from integration import run_tests
    run_tests(LinuxLVMTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import
import os
import shutil
import tempfile

# Import Salt Testing Libs
from salt.runners import winrepo
from salttesting import skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import patch, NO_MOCK, NO_MOCK_REASON

ensure_in_syspath('../../')
# Import Salt Libs
import integration

_WINREPO_SLS = r'''
winscp_x86:
  5.7.5:
    full_name: 'WinSCP 5.7.5'
    installer: 'http://heanet.dl.sourceforge.net/project/winscp/WinSCP/5.7.5/winscp575setup.exe'
    install_flags: '/SP- /verysilent /norestart'
    uninstaller: '%PROGRAMFILES%\WinSCP\unins000.exe'
    uninstall_flags: '/verysilent'
    msiexec: False
    locale: en_US
    reboot: False
  5.7.4:
    full_name: 'WinSCP 5.7.4'
    installer: 'http://cznic.dl.sourceforge.net/project/winscp/WinSCP/5.7.4/winscp574setup.exe'
    install_flags: '/SP- /verysilent /norestart'
    uninstaller: '%PROGRAMFILES%\WinSCP\unins000.exe'
    uninstall_flags: '/verysilent'
    msiexec: False
    locale: en_US
    reboot: False
'''

_WINREPO_GENREPO_DATA = {
    'name_map': {
        'WinSCP 5.7.4': 'winscp_x86',
        'WinSCP 5.7.5': 'winscp_x86'
    },
    'repo': {
        'winscp_x86': {
            '5.7.5': {
                'full_name': 'WinSCP 5.7.5',
                'installer': 'http://heanet.dl.sourceforge.net/project/winscp/WinSCP/5.7.5/winscp575setup.exe',
                'install_flags': '/SP- /verysilent /norestart',
                'uninstaller': '%PROGRAMFILES%\\WinSCP\\unins000.exe',
                'uninstall_flags': '/verysilent',
                'msiexec': False,
                'locale': 'en_US',
                'reboot': False
            },
            '5.7.4': {
                'full_name': 'WinSCP 5.7.4',
                'installer': 'http://cznic.dl.sourceforge.net/project/winscp/WinSCP/5.7.4/winscp574setup.exe',
                'install_flags': '/SP- /verysilent /norestart',
                'uninstaller': '%PROGRAMFILES%\\WinSCP\\unins000.exe',
                'uninstall_flags': '/verysilent',
                'msiexec': False,
                'locale': 'en_US',
                'reboot': False
            }
        }
    }
}

winrepo.__opts__ = {
    'winrepo_cachefile': 'winrepo.p',
    'renderer': 'yaml'
}
winrepo.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinrepoTest(integration.ShellCase):
    '''
    Test the winrepo runner
    '''
    def setUp(self):
        super(WinrepoTest, self).setUp()
        self.winrepo_dir = tempfile.mkdtemp(dir=integration.TMP)
        self.addCleanup(shutil.rmtree, self.winrepo_dir, ignore_errors=True)
        self.extmods_dir = tempfile.mkdtemp(dir=integration.TMP)
        self.addCleanup(shutil.rmtree, self.extmods_dir, ignore_errors=True)
        self.winrepo_sls_dir = os.path.join(self.winrepo_dir, 'repo_sls')
        os.mkdir(self.winrepo_sls_dir)
        self.maxDiff = None

    def test_genrepo(self):
        '''
        Test winrepo.genrepo runner
        '''
        sls_file = os.path.join(self.winrepo_sls_dir, 'wireshark.sls')
        # Add a winrepo SLS file
        with open(sls_file, 'w') as fp_:
            fp_.write(_WINREPO_SLS)
        with patch.dict(
                winrepo.__opts__,
                {'file_roots': {'base': [self.winrepo_dir]},
                 'winrepo_dir': self.winrepo_dir,
                 'extension_modules': self.extmods_dir}):
            self.assertEqual(winrepo.genrepo(), _WINREPO_GENREPO_DATA)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(WinrepoTest)



# -*- coding: utf-8 -*-
'''
Interaction with the Supervisor daemon
======================================

.. code-block:: yaml

    wsgi_server:
      supervisord.running:
        - require:
          - pkg: supervisor
        - watch:
          - file: /etc/nginx/sites-enabled/wsgi_server.conf
'''
from __future__ import absolute_import

# Import python libs
import logging
import salt.ext.six as six


log = logging.getLogger(__name__)


def _check_error(result, success_message):
    ret = {}

    if 'ERROR' in result:
        if 'already started' in result:
            ret['comment'] = success_message
        elif 'not running' in result:
            ret['comment'] = success_message
        else:
            ret['comment'] = result
            ret['result'] = False
    else:
        ret['comment'] = success_message

    return ret


def _is_stopped_state(state):
    if state in ('STOPPED', 'STOPPING', 'EXITED', 'FATAL', 'BACKOFF'):
        return True
    if state in ('STARTING', 'RUNNING'):
        return False
    return False


def running(name,
            restart=False,
            update=False,
            user=None,
            conf_file=None,
            bin_env=None):
    '''
    Ensure the named service is running.

    name
        Service name as defined in the supervisor configuration file

    restart
        Whether to force a restart

    update
        Whether to update the supervisor configuration.

    user
        Name of the user to run the supervisorctl command

        .. versionadded:: 0.17.0

    conf_file
        path to supervisorctl config file

    bin_env
        path to supervisorctl bin or path to virtualenv with supervisor
        installed

    '''
    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}

    if 'supervisord.status' not in __salt__:
        ret['result'] = False
        ret['comment'] = 'Supervisord module not activated. Do you need to install supervisord?'
        return ret

    all_processes = __salt__['supervisord.status'](
        user=user,
        conf_file=conf_file,
        bin_env=bin_env
    )

    # parse process groups
    process_groups = set()
    for proc in all_processes:
        if ':' in proc:
            process_groups.add(proc[:proc.index(':') + 1])
    process_groups = sorted(process_groups)

    matches = {}
    if name in all_processes:
        matches[name] = (all_processes[name]['state'].lower() == 'running')
    elif name in process_groups:
        for process in (x for x in all_processes if x.startswith(name)):
            matches[process] = (
                all_processes[process]['state'].lower() == 'running'
            )
    to_add = not bool(matches)

    if __opts__['test']:
        if not to_add:
            # Process/group already present, check if any need to be started
            to_start = [x for x, y in six.iteritems(matches) if y is False]
            if to_start:
                ret['result'] = None
                if name.endswith(':'):
                    # Process group
                    if len(to_start) == len(matches):
                        ret['comment'] = (
                            'All services in group {0!r} will be started'
                            .format(name)
                        )
                    else:
                        ret['comment'] = (
                            'The following services will be started: {0}'
                            .format(' '.join(to_start))
                        )
                else:
                    # Single program
                    ret['comment'] = 'Service {0} will be started'.format(name)
            else:
                if name.endswith(':'):
                    # Process group
                    ret['comment'] = (
                        'All services in group {0!r} are already running'
                        .format(name)
                    )
                else:
                    ret['comment'] = ('Service {0} is already running'
                                      .format(name))
        else:
            ret['result'] = None
            # Process/group needs to be added
            if name.endswith(':'):
                _type = 'Group {0!r}'.format(name)
            else:
                _type = 'Service {0}'.format(name)
            ret['comment'] = '{0} will be added and started'.format(_type)
        return ret

    changes = []
    just_updated = False
    if to_add:
        comment = 'Adding service: {0}'.format(name)
        __salt__['supervisord.reread'](
            user=user,
            conf_file=conf_file,
            bin_env=bin_env
        )
        result = __salt__['supervisord.add'](
            name,
            user=user,
            conf_file=conf_file,
            bin_env=bin_env
        )

        ret.update(_check_error(result, comment))
        changes.append(comment)
        log.debug(comment)

    elif update:
        comment = 'Updating supervisor'
        result = __salt__['supervisord.update'](
            user=user,
            conf_file=conf_file,
            bin_env=bin_env
        )
        ret.update(_check_error(result, comment))
        log.debug(comment)

        if '{0}: updated'.format(name) in result:
            just_updated = True

    is_stopped = None

    process_type = None
    if name in process_groups:
        process_type = 'group'

        # check if any processes in this group are stopped
        is_stopped = False
        for proc in all_processes:
            if proc.startswith(name) \
                    and _is_stopped_state(all_processes[proc]['state']):
                is_stopped = True
                break

    elif name in all_processes:
        process_type = 'service'

        if _is_stopped_state(all_processes[name]['state']):
            is_stopped = True
        else:
            is_stopped = False

    if is_stopped is False:
        if restart and not just_updated:
            comment = 'Restarting{0}: {1}'.format(
                process_type is not None and ' {0}'.format(process_type) or '',
                name
            )
            log.debug(comment)
            result = __salt__['supervisord.restart'](
                name,
                user=user,
                conf_file=conf_file,
                bin_env=bin_env
            )
            ret.update(_check_error(result, comment))
            changes.append(comment)
        elif just_updated:
            comment = 'Not starting updated{0}: {1}'.format(
                process_type is not None and ' {0}'.format(process_type) or '',
                name
            )
            result = comment
            ret.update({'comment': comment})
        else:
            comment = 'Not starting already running{0}: {1}'.format(
                process_type is not None and ' {0}'.format(process_type) or '',
                name
            )
            result = comment
            ret.update({'comment': comment})

    elif not just_updated:
        comment = 'Starting{0}: {1}'.format(
            process_type is not None and ' {0}'.format(process_type) or '',
            name
        )
        changes.append(comment)
        log.debug(comment)
        result = __salt__['supervisord.start'](
            name,
            user=user,
            conf_file=conf_file,
            bin_env=bin_env
        )

        ret.update(_check_error(result, comment))
        log.debug(six.text_type(result))

    if ret['result'] and len(changes):
        ret['changes'][name] = ' '.join(changes)
    return ret


def dead(name,
         user=None,
         conf_file=None,
         bin_env=None):
    '''
    Ensure the named service is dead (not running).

    name
        Service name as defined in the supervisor configuration file

    user
        Name of the user to run the supervisorctl command

        .. versionadded:: 0.17.0

    conf_file
        path to supervisorctl config file

    bin_env
        path to supervisorctl bin or path to virtualenv with supervisor
        installed

    '''
    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}

    if __opts__['test']:
        ret['result'] = None
        ret['comment'] = (
            'Service {0} is set to be stopped'.format(name))
    else:
        comment = 'Stopping service: {0}'.format(name)
        log.debug(comment)

        all_processes = __salt__['supervisord.status'](
            user=user,
            conf_file=conf_file,
            bin_env=bin_env
        )

        # parse process groups
        process_groups = []
        for proc in all_processes:
            if ':' in proc:
                process_groups.append(proc[:proc.index(':') + 1])
        process_groups = list(set(process_groups))

        is_stopped = None

        if name in process_groups:
            # check if any processes in this group are stopped
            is_stopped = False
            for proc in all_processes:
                if proc.startswith(name) \
                        and _is_stopped_state(all_processes[proc]['state']):
                    is_stopped = True
                    break

        elif name in all_processes:
            if _is_stopped_state(all_processes[name]['state']):
                is_stopped = True
            else:
                is_stopped = False
        else:
            # process name doesn't exist
            ret['comment'] = "Service {0} doesn't exist".format(name)

        if is_stopped is True:
            ret['comment'] = "Service {0} is not running".format(name)
        else:
            result = {name: __salt__['supervisord.stop'](
                name,
                user=user,
                conf_file=conf_file,
                bin_env=bin_env
            )}
            ret.update(_check_error(result, comment))
            ret['changes'][name] = comment
            log.debug(six.text_type(result))
    return ret


def mod_watch(name,
              restart=True,
              update=False,
              user=None,
              conf_file=None,
              bin_env=None,
              **kwargs):
    # Always restart on watch
    return running(
        name,
        restart=restart,
        update=update,
        user=user,
        conf_file=conf_file,
        bin_env=bin_env
    )



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
import os
import os.path
import tempfile

# Import Salt Testing libs
from salttesting import TestCase
from salttesting.mock import patch, MagicMock
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../')

# Import Salt libs
import integration
import salt.config
from salt.state import HighState
from salt.utils.odict import OrderedDict, DefaultOrderedDict


class HighStateTestCase(TestCase):
    def setUp(self):
        self.root_dir = tempfile.mkdtemp(dir=integration.TMP)
        self.state_tree_dir = os.path.join(self.root_dir, 'state_tree')
        self.cache_dir = os.path.join(self.root_dir, 'cachedir')
        if not os.path.isdir(self.root_dir):
            os.makedirs(self.root_dir)

        if not os.path.isdir(self.state_tree_dir):
            os.makedirs(self.state_tree_dir)

        if not os.path.isdir(self.cache_dir):
            os.makedirs(self.cache_dir)
        self.config = salt.config.minion_config(None)
        self.config['root_dir'] = self.root_dir
        self.config['state_events'] = False
        self.config['id'] = 'match'
        self.config['file_client'] = 'local'
        self.config['file_roots'] = dict(base=[self.state_tree_dir])
        self.config['cachedir'] = self.cache_dir
        self.config['test'] = False
        self.highstate = HighState(self.config)
        self.highstate.push_active()

    def tearDown(self):
        self.highstate.pop_active()

    def test_top_matches_with_list(self):
        top = {'env': {'match': ['state1', 'state2'], 'nomatch': ['state3']}}
        matches = self.highstate.top_matches(top)
        self.assertEqual(matches, {'env': ['state1', 'state2']})

    def test_top_matches_with_string(self):
        top = {'env': {'match': 'state1', 'nomatch': 'state2'}}
        matches = self.highstate.top_matches(top)
        self.assertEqual(matches, {'env': ['state1']})

    def test_matches_whitelist(self):
        matches = {'env': ['state1', 'state2', 'state3']}
        matches = self.highstate.matches_whitelist(matches, ['state2'])
        self.assertEqual(matches, {'env': ['state2']})

    def test_matches_whitelist_with_string(self):
        matches = {'env': ['state1', 'state2', 'state3']}
        matches = self.highstate.matches_whitelist(matches,
                                                   'state2,state3')
        self.assertEqual(matches, {'env': ['state2', 'state3']})


class TopFileMergeTestCase(TestCase):
    '''
    Test various merge strategies for multiple tops files collected from
    multiple environments. Various options correspond to merge strategies
    which can be set by the user with the top_file_merging_strategy config
    option.

    Refs #12483
    '''
    def setUp(self):
        '''
        Create multiple top files for use in each test
        '''
        self.env1 = {'base': {'*': ['e1_a', 'e1_b', 'e1_c']}}
        self.env2 = {'base': {'*': ['e2_a', 'e2_b', 'e2_c']}}
        self.env3 = {'base': {'*': ['e3_a', 'e3_b', 'e3_c']}}
        self.config = self._make_default_config()
        self.highstate = HighState(self.config)

    def _make_default_config(self):
        config = salt.config.minion_config(None)
        root_dir = tempfile.mkdtemp(dir=integration.TMP)
        state_tree_dir = os.path.join(root_dir, 'state_tree')
        cache_dir = os.path.join(root_dir, 'cachedir')
        config['root_dir'] = root_dir
        config['state_events'] = False
        config['id'] = 'match'
        config['file_client'] = 'local'
        config['file_roots'] = dict(base=[state_tree_dir])
        config['cachedir'] = cache_dir
        config['test'] = False
        return config

    def _get_tops(self):
        '''
        A test helper to emulate HighState.get_tops() but just to construct
        an appropriate data structure for top files from multiple environments
        '''
        tops = DefaultOrderedDict(list)

        tops['a'].append(self.env1)
        tops['b'].append(self.env2)
        tops['c'].append(self.env3)
        return tops

    def test_basic_merge(self):
        '''
        This is the default approach for Salt. Merge the top files with the
        earlier appends taking precendence. Since Salt does the appends
        lexecographically, this is effectively a test against the default
        lexecographical behaviour.
        '''
        merged_tops = self.highstate.merge_tops(self._get_tops())

        expected_merge = DefaultOrderedDict(OrderedDict)
        expected_merge['base']['*'] = ['e1_c', 'e1_b', 'e1_a']
        self.assertEqual(merged_tops, expected_merge)

    def test_merge_strategy_same(self):
        '''
        Test to see if the top file that corresponds
        to the requested env is the one that is used
        by the state system
        '''
        config = self._make_default_config()
        config['top_file_merging_strategy'] = 'same'
        config['environment'] = 'b'
        highstate = HighState(config)
        ret = highstate.get_tops()
        self.assertEqual(ret, OrderedDict([('b', [{}])]))

    def test_ordered_merge(self):
        '''
        Test to see if the merger respects environment
        ordering
        '''
        config = self._make_default_config()
        config['top_file_merging_strategy'] = 'merge'
        config['env_order'] = ['b', 'a', 'c']
        with patch('salt.fileclient.FSClient.envs', MagicMock(return_value=['a', 'b', 'c'])):
            highstate = HighState(config)
            ret = highstate.get_tops()
        self.assertEqual(ret, OrderedDict([('a', [{}]), ('c', [{}]), ('b', [{}])]))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(HighStateTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import localemod

# Globals
localemod.__grains__ = {}
localemod.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LocalemodTestCase(TestCase):
    '''
    Test cases for salt.modules.localemod
    '''
    def test_list_avail(self):
        '''
        Test for Lists available (compiled) locales
        '''
        with patch.dict(localemod.__salt__,
                        {'cmd.run': MagicMock(return_value='A\nB')}):
            self.assertEqual(localemod.list_avail(), ['A', 'B'])

    def test_get_locale(self):
        '''
        Test for Get the current system locale
        '''
        with patch.dict(localemod.__grains__, {'os_family': ['Arch']}):
            with patch.object(localemod, '_locale_get', return_value=True):
                self.assertTrue(localemod.get_locale())

        with patch.dict(localemod.__grains__, {'os_family': ['Gentoo']}):
            with patch.dict(localemod.__salt__, {'cmd.run':
                                                 MagicMock(return_value='A')}):
                self.assertEqual(localemod.get_locale(), 'A')

        with patch.dict(localemod.__grains__, {'os_family': ['A']}):
            with patch.dict(localemod.__salt__,
                            {'cmd.run': MagicMock(return_value='A=B')}):
                self.assertEqual(localemod.get_locale(), 'B')

        with patch.dict(localemod.__grains__, {'os_family': ['A']}):
            with patch.dict(localemod.__salt__, {'cmd.run':
                                                 MagicMock(return_value='A')}):
                self.assertEqual(localemod.get_locale(), '')

    def test_set_locale(self):
        '''
        Test for Sets the current system locale
        '''
        with patch.dict(localemod.__grains__, {'os_family': ['Arch']}):
            with patch.object(localemod, '_localectl_set', return_value=True):
                self.assertTrue(localemod.set_locale('l'))

        with patch.dict(localemod.__grains__, {'os_family': ['Gentoo']}):
            with patch.dict(localemod.__salt__, {'cmd.retcode':
                                                 MagicMock(return_value='A')}):
                self.assertFalse(localemod.set_locale('l'))

        with patch.dict(localemod.__grains__, {'os_family': ['A']}):
            self.assertTrue(localemod.set_locale('locale'))

    def test_avail(self):
        '''
        Test for Check if a locale is available
        '''
        with patch('salt.utils.locales.normalize_locale',
                   MagicMock(return_value='en_US.UTF-8 UTF-8')):
            with patch.dict(localemod.__salt__,
                            {'locale.list_avail':
                             MagicMock(return_value=['A', 'B'])}):
                self.assertTrue(localemod.avail('locale'))

    @patch('salt.utils.which', MagicMock(return_value='/some/dir/path'))
    def test_gen_locale_not_valid(self):
        '''
        Tests the return of gen_locale when the provided locale is not found
        '''
        with patch.dict(localemod.__grains__, {'os': 'Debian'}):
            with patch.dict(localemod.__salt__,
                            {'file.search': MagicMock(return_value=False)}):
                self.assertFalse(localemod.gen_locale('foo'))

    @patch('salt.utils.which', MagicMock(return_value='/some/dir/path'))
    def test_gen_locale_debian(self):
        '''
        Tests the return of successful gen_locale on Debian system
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os': 'Debian'}):
            with patch.dict(localemod.__salt__,
                            {'file.search': MagicMock(return_value=True),
                             'file.replace': MagicMock(return_value=True),
                             'cmd.run_all': MagicMock(return_value=ret)}):
                self.assertTrue(localemod.gen_locale('en_US.UTF-8 UTF-8'))

    @patch('salt.utils.which', MagicMock(return_value='/some/dir/path'))
    def test_gen_locale_debian_no_charmap(self):
        '''
        Tests the return of successful gen_locale on Debian system without a charmap
        '''
        def file_search(search, pattern, flags):
            '''
            mock file.search
            '''
            if len(pattern.split()) == 1:
                return False
            else:  # charmap was supplied
                return True

        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os': 'Debian'}):
            with patch.dict(localemod.__salt__,
                            {'file.search': file_search,
                             'file.replace': MagicMock(return_value=True),
                             'cmd.run_all': MagicMock(return_value=ret)}):
                self.assertTrue(localemod.gen_locale('en_US.UTF-8'))

    @patch('salt.utils.which', MagicMock(return_value='/some/dir/path'))
    @patch('os.listdir', MagicMock(return_value=['en_US']))
    def test_gen_locale_ubuntu(self):
        '''
        Test the return of successful gen_locale on Ubuntu system
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__salt__,
                        {'file.replace': MagicMock(return_value=True),
                         'file.touch': MagicMock(return_value=None),
                         'file.append': MagicMock(return_value=None),
                         'cmd.run_all': MagicMock(return_value=ret)}):
            with patch.dict(localemod.__grains__, {'os': 'Ubuntu'}):
                self.assertTrue(localemod.gen_locale('en_US.UTF-8'))

    @patch('os.listdir', MagicMock(return_value=['en_US.UTF-8']))
    def test_gen_locale_gentoo(self):
        '''
        Tests the return of successful gen_locale on Gentoo system
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os_family': 'Gentoo'}):
            with patch.dict(localemod.__salt__,
                            {'file.search': MagicMock(return_value=True),
                             'file.replace': MagicMock(return_value=True),
                             'cmd.run_all': MagicMock(return_value=ret)}):
                self.assertTrue(localemod.gen_locale('en_US.UTF-8 UTF-8'))

    @patch('os.listdir', MagicMock(return_value=['en_US.UTF-8']))
    def test_gen_locale_gentoo_no_charmap(self):
        '''
        Tests the return of successful gen_locale on Gentoo system without a charmap
        '''
        def file_search(search, pattern, flags):
            '''
            mock file.search
            '''
            if len(pattern.split()) == 1:
                return False
            else:  # charmap was supplied
                return True

        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__grains__, {'os_family': 'Gentoo'}):
            with patch.dict(localemod.__salt__,
                            {'file.search': file_search,
                             'file.replace': MagicMock(return_value=True),
                             'cmd.run_all': MagicMock(return_value=ret)}):
                self.assertTrue(localemod.gen_locale('en_US.UTF-8'))

    @patch('salt.utils.which', MagicMock(return_value='/some/dir/path'))
    @patch('os.listdir', MagicMock(return_value=['en_US']))
    def test_gen_locale(self):
        '''
        Tests the return of successful gen_locale
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__salt__,
                        {'cmd.run_all': MagicMock(return_value=ret),
                         'file.replace': MagicMock()}):
            self.assertTrue(localemod.gen_locale('en_US.UTF-8'))

    @patch('salt.utils.which', MagicMock(return_value='/some/dir/path'))
    @patch('os.listdir', MagicMock(return_value=['en_US']))
    def test_gen_locale_verbose(self):
        '''
        Tests the return of successful gen_locale
        '''
        ret = {'stdout': 'saltines', 'stderr': 'biscuits', 'retcode': 0, 'pid': 1337}
        with patch.dict(localemod.__salt__,
                        {'cmd.run_all': MagicMock(return_value=ret),
                         'file.replace': MagicMock()}):
            self.assertEqual(localemod.gen_locale('en_US.UTF-8', verbose=True), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(LocalemodTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
runit service module

This module is compatible with the :mod:`service <salt.states.service>` states,
so it can be used to maintain services using the ``provider`` argument:

.. code-block:: yaml

    myservice:
      service:
        - running
        - provider: runit

Note that the ``enabled`` argument is not available with this provider.
'''
from __future__ import absolute_import

# Import python libs
import os
import re

# Import salt libs
from salt.exceptions import CommandExecutionError

__func_alias__ = {
    'reload_': 'reload'
}

VALID_SERVICE_DIRS = [
    '/service',
    '/var/service',
    '/etc/service',
]
SERVICE_DIR = None
for service_dir in VALID_SERVICE_DIRS:
    if os.path.exists(service_dir):
        SERVICE_DIR = service_dir
        break


def _service_path(name):
    '''
    build service path
    '''
    if not SERVICE_DIR:
        raise CommandExecutionError("Could not find service directory.")
    return '{0}/{1}'.format(SERVICE_DIR, name)


def start(name):
    '''
    Starts service via runit

    CLI Example:

    .. code-block:: bash

        salt '*' runit.start <service name>
    '''
    cmd = 'sv start {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


def stop(name):
    '''
    Stops service via runit

    CLI Example:

    .. code-block:: bash

        salt '*' runit.stop <service name>
    '''
    cmd = 'sv stop {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


def term(name):
    '''
    Send a TERM to service via runit

    CLI Example:

    .. code-block:: bash

        salt '*' runit.term <service name>
    '''
    cmd = 'sv term {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


def reload_(name):
    '''
    Send a HUP to service via runit

    CLI Example:

    .. code-block:: bash

        salt '*' runit.reload <service name>
    '''
    cmd = 'sv reload {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


def restart(name):
    '''
    Restart service via runit. This will stop/start service

    CLI Example:

    .. code-block:: bash

        salt '*' runit.restart <service name>
    '''
    cmd = 'sv restart {0}'.format(_service_path(name))
    return not __salt__['cmd.retcode'](cmd)


def full_restart(name):
    '''
    Calls runit.restart() function

    CLI Example:

    .. code-block:: bash

        salt '*' runit.full_restart <service name>
    '''
    restart(name)


def status(name, sig=None):
    '''
    Return the status for a service via runit, return pid if running

    CLI Example:

    .. code-block:: bash

        salt '*' runit.status <service name>
    '''
    cmd = 'sv status {0}'.format(_service_path(name))
    out = __salt__['cmd.run_stdout'](cmd)
    try:
        pid = re.search(r'{0}: \(pid (\d+)\)'.format(name), out).group(1)
    except AttributeError:
        pid = ''
    return pid


def available(name):
    '''
    Returns ``True`` if the specified service is available, otherwise returns
    ``False``.

    CLI Example:

    .. code-block:: bash

        salt '*' runit.available foo
    '''
    return name in get_all()


def missing(name):
    '''
    The inverse of runit.available.
    Returns ``True`` if the specified service is not available, otherwise returns
    ``False``.

    CLI Example:

    .. code-block:: bash

        salt '*' runit.missing foo
    '''
    return name not in get_all()


def get_all():
    '''
    Return a list of all available services

    CLI Example:

    .. code-block:: bash

        salt '*' runit.get_all
    '''
    if not SERVICE_DIR:
        raise CommandExecutionError("Could not find service directory.")
    return sorted(os.listdir(SERVICE_DIR))



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import varnish

# Globals
varnish.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class VarnishTestCase(TestCase):
    '''
    Test cases for salt.modules.varnish
    '''
    def test_version(self):
        '''
        Test to return server version from varnishd -V
        '''
        with patch.dict(varnish.__salt__,
                        {'cmd.run': MagicMock(return_value='(varnish-2.0)')}):
            self.assertEqual(varnish.version(), '2.0')

    def test_ban(self):
        '''
        Test to add ban to the varnish cache
        '''
        with patch.object(varnish, '_run_varnishadm',
                          return_value={'retcode': 0}):
            self.assertTrue(varnish.ban('ban_expression'))

    def test_ban_list(self):
        '''
        Test to list varnish cache current bans
        '''
        with patch.object(varnish, '_run_varnishadm',
                          return_value={'retcode': True}):
            self.assertFalse(varnish.ban_list())

        with patch.object(varnish, '_run_varnishadm',
                          return_value={'retcode': False,
                                        'stdout': 'A\nB\nC'}):
            self.assertEqual(varnish.ban_list(), ['B', 'C'])

    def test_purge(self):
        '''
        Test to purge the varnish cache
        '''
        with patch.object(varnish, 'ban', return_value=True):
            self.assertTrue(varnish.purge())

    def test_param_set(self):
        '''
        Test to set a param in varnish cache
        '''
        with patch.object(varnish, '_run_varnishadm',
                          return_value={'retcode': 0}):
            self.assertTrue(varnish.param_set('param', 'value'))

    def test_param_show(self):
        '''
        Test to show params of varnish cache
        '''
        with patch.object(varnish, '_run_varnishadm',
                          return_value={'retcode': True,
                                        'stdout': 'A\nB\nC'}):
            self.assertFalse(varnish.param_show('param'))

        with patch.object(varnish, '_run_varnishadm',
                          return_value={'retcode': False,
                                        'stdout': 'A .1\nB .2\n'}):
            self.assertEqual(varnish.param_show('param'), {'A': '.1'})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(VarnishTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
unit tests for the alias state
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import alias

alias.__opts__ = {}
alias.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class AliasTest(TestCase):
    '''
    Validate the alias state
    '''
    def test_present_has_target(self):
        '''
        test alias.present has target already
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Alias {0} already present'.format(name),
               'changes': {},
               'name': name,
               'result': True}

        has_target = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {'aliases.has_target': has_target}):
            self.assertEqual(alias.present(name, target), ret)

    def test_present_has_not_target_test(self):
        '''
        test alias.present has't got target yet test mode
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Alias {0} -> {1} is set to be added'.format(name, target),
               'changes': {},
               'name': name,
               'result': None}

        has_target = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {'aliases.has_target': has_target}):
            with patch.dict(alias.__opts__, {'test': True}):
                self.assertEqual(alias.present(name, target), ret)

    def test_present_set_target(self):
        '''
        test alias.present set target
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Set email alias {0} -> {1}'.format(name, target),
               'changes': {'alias': name},
               'name': name,
               'result': True}

        has_target = MagicMock(return_value=False)
        set_target = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {'aliases.has_target': has_target}):
            with patch.dict(alias.__opts__, {'test': False}):
                with patch.dict(alias.__salt__, {'aliases.set_target': set_target}):
                    self.assertEqual(alias.present(name, target), ret)

    def test_present_set_target_failed(self):
        '''
        test alias.present set target failure
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Failed to set alias {0} -> {1}'.format(name, target),
               'changes': {},
               'name': name,
               'result': False}

        has_target = MagicMock(return_value=False)
        set_target = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {'aliases.has_target': has_target}):
            with patch.dict(alias.__opts__, {'test': False}):
                with patch.dict(alias.__salt__, {'aliases.set_target': set_target}):
                    self.assertEqual(alias.present(name, target), ret)

    def test_absent_already_gone(self):
        '''
        test alias.absent already gone
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Alias {0} already absent'.format(name),
               'changes': {},
               'name': name,
               'result': True}

        get_target = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {'aliases.get_target': get_target}):
            self.assertEqual(alias.absent(name), ret)

    def test_absent_not_gone_test(self):
        '''
        test alias.absent already gone test mode
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Alias {0} is set to be removed'.format(name),
               'changes': {},
               'name': name,
               'result': None}

        get_target = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {'aliases.get_target': get_target}):
            with patch.dict(alias.__opts__, {'test': True}):
                self.assertEqual(alias.absent(name), ret)

    def test_absent_rm_alias(self):
        '''
        test alias.absent remove alias
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Removed alias {0}'.format(name),
               'changes': {'alias': name},
               'name': name,
               'result': True}

        get_target = MagicMock(return_value=True)
        rm_alias = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {'aliases.get_target': get_target}):
            with patch.dict(alias.__opts__, {'test': False}):
                with patch.dict(alias.__salt__, {'aliases.rm_alias': rm_alias}):
                    self.assertEqual(alias.absent(name), ret)

    def test_absent_rm_alias_failed(self):
        '''
        test alias.absent remove alias failure
        '''
        name = 'saltdude'
        target = 'dude@saltstack.com'
        ret = {'comment': 'Failed to remove alias {0}'.format(name),
               'changes': {},
               'name': name,
               'result': False}

        get_target = MagicMock(return_value=True)
        rm_alias = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {'aliases.get_target': get_target}):
            with patch.dict(alias.__opts__, {'test': False}):
                with patch.dict(alias.__salt__, {'aliases.rm_alias': rm_alias}):
                    self.assertEqual(alias.absent(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(AliasTest, needs_daemon=False)
